import communication.ConnectionUtil;
import communication.DataUtil;
import controlP5.*;
import controlP5.Button;
import processing.core.PApplet;
import processing.core.PFont;
import processing.core.PVector;

import java.awt.*;

/**
 * Created by dalind on 29/05/15.
 */
public class SCPhone extends PApplet {

    // toggle serial IO usage
    boolean offline = false;
    ConnectionUtil connectionUtil;
    DataUtil dataUtil;

    ControlP5 cp5;

    DropdownList serialList;
    private Toggle connectButton;
    private Button disconnectButton;

    boolean fullscreen = false;

    boolean slidersActive = true;
    boolean initFinished = false;

    int sliderValueMotorAll = 0;
    int sliderValueMotorCoupled = 0;

    static int NUMBER_MOTORS = 6;

    int sliderValueMotor1 = 0;
    int sliderValueMotor2 = 0;
    int sliderValueMotor3 = 0;
    int sliderValueMotor4 = 0;
    int sliderValueMotor5 = 0;
    int sliderValueMotor6 = 0;

    boolean[] activeMotors = {false, false, false, false, false, false};

    private Textfield inputTextfield;

    public boolean sketchFullScreen() {
        return fullscreen;
    }

    public void setup() {
        if (fullscreen) {
            size(displayWidth, displayHeight);
        } else
        {
            size(640,480);
        }

        cp5 = new ControlP5(this);
        connectionUtil = new ConnectionUtil(this);
        dataUtil = new DataUtil(this, connectionUtil);

        createSliders();
        createTextInput();
        createPortSelection();

        initFinished = true;
    }

    private void createTextInput() {

        PFont font = createFont("arial",15);

        inputTextfield = new Textfield(cp5, "input")
                .setPosition(50,75)
                .setSize(100,30)
                .setFont(font)
                .setFocus(true)
                //.setColor(color(255,0,0))
        ;

        cp5.addButton("send")
                .setPosition(165,75)
                .setSize(50,30);
    }

    private void createSliders()
    {
        PVector position = new PVector (50,150);
        PVector offset = new PVector(70,0);

        cp5.addSlider("sliderValueMotorAll")
                .setCaptionLabel("Motor All")
                .setPosition(position)
                .setRange(0, 99)
                .setSize(50, 150)
        ;
        position.add(offset);

        PVector togglePosition = new PVector(position.x,position.y + 170);

        for(int i = 1; i <= NUMBER_MOTORS; i++)
        {
            cp5.addSlider("sliderValueMotor"+i)
                    .setCaptionLabel("Motor "+i)
                    .setPosition(position)
                    .setRange(0, 99)
                    .setSize(50, 150)
            ;
            position.add(offset);

            cp5.addToggle("Motor " + i)
                    .setCaptionLabel("     Couple")
                    .setPosition(togglePosition)
                    .setSize(50,50);
            togglePosition.add(offset);
        }
    }

    public void draw() {
        background(128);

        if(!offline)
            dataUtil.receiveData();

    }

    private void createPortSelection()
    {
        createPortList();

        connectButton = new Toggle(cp5, "Connect")
                .setValue(0)
                .setPosition(270, 23)
                .setSize(75, 25);
        connectButton.getCaptionLabel().getStyle().setMarginTop(-21);
        connectButton.getCaptionLabel().getStyle().setMarginLeft(4);

        disconnectButton = new Button(cp5, "Disconnect")
                .setValue(0)
                .setPosition(360,23)
                .setSize(75,25)
                .setLock(true)
                .setColorActive(155);
    }

    private void createPortList()
    {
        serialList = cp5.addDropdownList("serial-ports")
                .setPosition(50, 50)
                .setSize(200,200);

        serialList.setBackgroundColor(color(190));
        serialList.setItemHeight(25);
        serialList.setBarHeight(25);
        serialList.captionLabel().set("Serial Ports");
        serialList.captionLabel().style().marginTop = 7;
        serialList.captionLabel().style().marginLeft = 3;
        serialList.valueLabel().style().marginTop = 3;

        String[] serialPorts = connectionUtil.getPortList();

        if(serialPorts.length < 1)
        {
            serialList.addItem("No ports found", 0);
        }

        for (int i=0;i<serialPorts.length;i++)
        {
            println(serialPorts[i]);
            serialList.addItem(serialPorts[i], i);
        }

        serialList.setColorBackground(color(60));
        serialList.setColorActive(color(255, 128));
    }

    public void controlEvent(ControlEvent theEvent)
    {
        if(!initFinished)
            return;

        if (theEvent.isGroup()) {
            println("event from group : " + theEvent.getGroup().getValue() + " from " + theEvent.getGroup());

            if (theEvent.getGroup() == serialList)
                connectionUtil.handlePortSelection((int) theEvent.getGroup().getValue());

        } else if (theEvent.isController()) {
            println("event from controller : " + theEvent.getController().getValue() + " from " + theEvent.getController());

            if(theEvent.getController() instanceof Slider && slidersActive)
                handleSliderEvent((Slider)theEvent.getController());

            if(theEvent.getController() instanceof Textfield ||
                    theEvent.getController().getName().equals("send"))
                handleTextInput();

            if(theEvent.getController() instanceof Toggle && !theEvent.getController().equals(connectButton))
            {
                handleToggleChanged(theEvent.getController().getName(), theEvent.getValue());
            }

            if (theEvent.getController().equals(connectButton) && connectButton.getState() == true
                    && theEvent.getController().getValue() > 0) {
                if(connectionUtil.isConnected == true)
                    return;

                boolean connected = connectionUtil.connect();

                if(!connected) {
                    connectButton.setState(false);
                }
                else {
                    connectButton.setState(true);
                    disconnectButton.setLock(false);
                }

                cp5.update();

            } else if (theEvent.getController().equals(disconnectButton)
                    || (theEvent.getController().equals(connectButton) && theEvent.getController().getValue() > 0)) {
                if(connectionUtil.isConnected == false)
                    return;

                connectionUtil.disconnect();

                if(connectButton.getState() != false) {
                    connectButton.setState(false);

                    disconnectButton.setLock(true);
                }

                cp5.update();
            }
        }
    }

    private void handleToggleChanged(String name, float value) {
        boolean enabled = value > 0.0;

        switch (name)
        {
            case "Motor 1":
                activeMotors[0] = enabled;
                break;
            case "Motor 2":
                activeMotors[1] = enabled;
                break;
            case "Motor 3":
                activeMotors[2] = enabled;
                break;
            case "Motor 4":
                activeMotors[3] = enabled;
                break;
            case "Motor 5":
                activeMotors[4] = enabled;
                break;
            case "Motor 6":
                activeMotors[5] = enabled;
                break;
        }
    }

    private void handleTextInput()
    {
        if(!connectionUtil.isConnected)
            return;

        String text = inputTextfield.getText();

        if(text.isEmpty())
            return;

        dataUtil.write(text);
    }

    private void handleSliderEvent(Slider slider)
    {
        switch (slider.getName())
        {
            case "sliderValueMotorAll":
                slidersActive = false;

                if(cp5.getController("sliderValueMotor1") != null)
                {
                    ((Slider)cp5.getController("sliderValueMotor1")).setValue(sliderValueMotorAll);
                    ((Slider)cp5.getController("sliderValueMotor2")).setValue(sliderValueMotorAll);
                    ((Slider)cp5.getController("sliderValueMotor3")).setValue(sliderValueMotorAll);
                    ((Slider)cp5.getController("sliderValueMotor4")).setValue(sliderValueMotorAll);
                    ((Slider)cp5.getController("sliderValueMotor5")).setValue(sliderValueMotorAll);
                    ((Slider)cp5.getController("sliderValueMotor6")).setValue(sliderValueMotorAll);
                }
                slidersActive = true;
                dataUtil.write(0,sliderValueMotorAll);
                break;
            case "sliderValueMotor1":
                dataUtil.write(1,sliderValueMotor1);
                updateCoupledMotors(1, sliderValueMotor1);
                break;
            case "sliderValueMotor2":
                dataUtil.write(2,sliderValueMotor2);
                updateCoupledMotors(2, sliderValueMotor2);
                break;
            case "sliderValueMotor3":
                dataUtil.write(3,sliderValueMotor3);
                updateCoupledMotors(3, sliderValueMotor3);
                break;
            case "sliderValueMotor4":
                dataUtil.write(4,sliderValueMotor4);
                updateCoupledMotors(4, sliderValueMotor4);
                break;
            case "sliderValueMotor5":
                dataUtil.write(5,sliderValueMotor5);
                updateCoupledMotors(5, sliderValueMotor5);
                break;
            case "sliderValueMotor6":
                dataUtil.write(6,sliderValueMotor6);
                updateCoupledMotors(6, sliderValueMotor6);
                break;

        }
    }

    private void updateCoupledMotors(int master, int value) {

        if(activeMotors[master - 1] == false)
            return;

        slidersActive = false;

        for (int i = 1; i <= activeMotors.length; i++)
        {
            if(!IsSame(master, i) && activeMotors[i - 1] == true)
            {
                dataUtil.write(i, value);
                ((Slider)cp5.getController("sliderValueMotor"+(i))).setValue(value);
            }
        }

        slidersActive = true;
    }

    private static boolean IsSame(int first, int second)
    {
        return Math.abs(first - second) < .0001;
    }
}
